#ifndef FLUX_SPECTRUMSTATS_HH
#define FLUX_SPECTRUMSTATS_HH

#include "SpectrumConstants.hh"

namespace Flux {

  /**
   * \brief Container for counts and exposure information in an energy / time bin.
   *
   * \author Heidelberg team
   *
   * \todo SpectrumStats should be a subclass of Tools::EventStats,
   * instead of copy & pasting code.
   * There is some subtle error handling in methods like SignificanceUnchecked
   * or Excess and copying will eventually lead to different behavior of
   * calling this method on a SpectrumStats and an EventStats, which is bad.
   *
   */
  class SpectrumStats {

  public: 
    
    SpectrumStats();
    inline virtual ~SpectrumStats() {}
    
    virtual void Clear();

    virtual double GetField(std::string field) const;
    virtual void SetField(std::string field, double value);

    bool Compare(const SpectrumStats& rhs) const;
    bool operator==(const SpectrumStats& rhs) const;
    bool operator!=(const SpectrumStats& rhs) const;
    void Add(const SpectrumStats& rhs);
    SpectrumStats& operator+=(const SpectrumStats& rhs);
    SpectrumStats operator+(const SpectrumStats& rhs) const;

    double Alpha() const;
    double AlphaUnchecked() const;
    double Significance() const; 
    double SignificanceUnchecked() const;
    double Excess() const;
    double ExcessError() const;
    double ExcessErrorUp() const;
    double ExcessErrorDown() const;
    double FracError() const;
    double FracErrorUp() const;
    double FracErrorDown() const;
    double ExcessSensitivity(double cl=DEFAULT_CL) const;
    void ExcessLimits(double& excess_down, double& excess_up, double cl=DEFAULT_CL) const;
    void ZeroExcessLimits(double& excess_down, double& excess_up, double cl=DEFAULT_CL) const;
    double Background() const;

    void SetPrintPrefix(std::string prefix){ fPrintPrefix = prefix; }
    virtual void print(std::ostream& os) const;

  public:
    // Counts
    int     nOn;		///< On region event count
    int     nOff;		///< Off region event count

    // Relative background exposure
    double  exposureOn;		///< On region hadron exposure
    double  exposureOff;	///< Off region hadron exposure

    // Gamma exposure
    double trueAreaTimeOn;	///< On region gamma true exposure
    double trueAreaTimeOff;	///< Off region gamma true exposure

    double recoAreaTimeOn;	///< On region gamma reco exposure
    double recoAreaTimeOff;	///< Off region gamma reco exposure

    // For EventWeighted flux point computation
    double liveTime;            ///< Livetime
    double offLiveTime;         ///< offLiveTime
    // The following two numbers are sums of inverse effective
    // areas of events collected in the given livetime.
    double invRecoAreaOn;	///< On region Sum(1/A_i)
    double invRecoAreaOff;	///< Off region Sum(1/A_i)

  protected:
    mutable std::string fPrintPrefix; //! not streamed, just for pretty printing

  private:
    ClassDef(Flux::SpectrumStats,2)
  };

}

std::ostream& operator<<( std::ostream& stream, const Flux::SpectrumStats&);

#endif
